package com.netmera.netmera_flutter_sdk;

import android.text.format.DateFormat;

import com.google.gson.Gson;
import com.netmera.NetmeraPushObject;
import com.netmera.NetmeraPushStyle;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * Copyright (c) 2019 Inomera Research.
 *
 * @author Burak Soykal
 */
class FNetmeraPushObject {

    static List<HashMap<String, Object>> mapPushObjects(List<NetmeraPushObject> pushObjects) {
        List<HashMap<String, Object>> pushObjectList = new ArrayList<>();
        for (NetmeraPushObject netmeraPushObject : pushObjects) {
            pushObjectList.add(getPushObjectMap(netmeraPushObject));
        }
        return pushObjectList;
    }

    static HashMap<String, Object> getPushObjectMap(NetmeraPushObject pushObject) {
        HashMap<String, Object> map = new HashMap<>();
        if (pushObject == null) {
            return map;
        }
        map.put("pushType", pushObject.getPushType());
        map.put("pushId", pushObject.getPushId());
        map.put("pushInstanceId", pushObject.getPushInstanceId());
        map.put("inboxStatus", pushObject.getInboxStatus());
        map.put("category", pushObject.getCategory());
        map.put("customJson", new Gson().fromJson(pushObject.getCustomJson(), HashMap.class));

        if (pushObject.getPopupExpirationTime() != null) {
            map.put("expireTime", String.valueOf(DateFormat.format("yyyy-MM-dd HH:mm:ss", pushObject.getPopupExpirationTime())));
        }

        if (pushObject.getDeepLink() != null) {
            map.put("action_deeplink_url", String.valueOf(pushObject.getDeepLink()));
        }

        if (pushObject.getExternalId() != null) {
            map.put("externalId", pushObject.getExternalId());
        }

        if (pushObject.getCategories() != null && !pushObject.getCategories().isEmpty()) {
            map.put("categories", pushObject.getCategories());
        }

        if (pushObject.getInteractiveActions() != null && !pushObject.getInteractiveActions().isEmpty()) {
            map.put("interactiveActions", new Gson().toJson(pushObject.getInteractiveActions()));
        }

        /* Push Style */
        if (pushObject.getPushStyle() != null) {
            NetmeraPushStyle pushStyle = pushObject.getPushStyle();
            if (pushStyle.getContentTitle() != null) {
                map.put("title", pushStyle.getContentTitle());
            }
            if (pushStyle.getSubText() != null) {
                map.put("subtitle", pushStyle.getSubText());
            }
            if (pushStyle.getContentText() != null) {
                map.put("body", pushStyle.getContentText());
            }
            if (pushStyle.getBigPicturePath() != null) {
                map.put("mediaAttachmentURL", pushStyle.getBigPicturePath());
            }
            if (pushStyle.retrieveCarouselObjects() != null) {
                map.put("carousel", pushStyle.retrieveCarouselObjects());
            }
        }

        if (pushObject.getSendDate() != null) {
            map.put("sendDate", String.valueOf(DateFormat.format("yyyy-MM-dd HH:mm:ss", pushObject.getSendDate())));
        }

        if(pushObject.getPushAction() !=null){
            map.put("pushAction", new Gson().fromJson(pushObject.getPushAction(), HashMap.class));

            if (pushObject.getWebPageUrl() != null) {
                map.put("action_webpage_url", pushObject.getWebPageUrl());
            }
        }

        return map;
    }

}
