package com.netmera.netmera_flutter_sdk;

import android.app.ActivityManager;
import android.app.KeyguardManager;
import android.content.Context;
import android.os.Bundle;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Copyright (c) 2023 Netmera Research.
 *
 * @author Initial Code
 */

public class FNetmeraUtils {

    /**
     * Returns if the application is currently in a state where user interaction is possible.
     * This method is called when a callback is triggered to determine how the callback should be handled.
     *
     * @param context context.
     * @return True if the application is currently in a state where user interaction is possible,
     * false otherwise.
     */
    static boolean isApplicationForeground(Context context) {
        KeyguardManager keyguardManager =
                (KeyguardManager) context.getSystemService(Context.KEYGUARD_SERVICE);

        if (keyguardManager != null && keyguardManager.isKeyguardLocked()) {
            return false;
        }

        ActivityManager activityManager =
                (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        if (activityManager == null) return false;

        List<ActivityManager.RunningAppProcessInfo> appProcesses =
                activityManager.getRunningAppProcesses();
        if (appProcesses == null) return false;

        final String packageName = context.getPackageName();
        for (ActivityManager.RunningAppProcessInfo appProcess : appProcesses) {
            if (appProcess.importance == ActivityManager.RunningAppProcessInfo.IMPORTANCE_FOREGROUND
                    && appProcess.processName.equals(packageName)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Used to convert Bundle objects to HashMap
     *
     * @param bundle Object of Bundle type to be converted.
     * @return The converted object of type HashMap
     */
    static Map<String, Object> convertBundleToMap(Bundle bundle) {
        Map<String, Object> map = new HashMap<>();
        if (bundle != null) {
            for (String key : bundle.keySet()) {
                Object value = bundle.get(key);
                map.put(key, value);
            }
        }
        return map;
    }

}
